package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.KnowledgeAgent;
import com.redxun.knowledge.entity.dto.KnowledgeAgentDto;
import com.redxun.knowledge.entity.vo.KnowledgeAgentVo;
import com.redxun.knowledge.mapper.KnowledgeAgentMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;

/**
 * 文件名: KnowledgeAgentServiceImpl
 * 作者: zizg
 * 时间: 2023/6/5
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Service
public class KnowledgeAgentServiceImpl extends SuperServiceImpl<KnowledgeAgentMapper, KnowledgeAgent> implements BaseService<KnowledgeAgent> {

    @Autowired
    private UserService userService;

    @Autowired
    private KnowledgeAgentMapper knowledgeAgentMapper;

    @Override
    public BaseDao<KnowledgeAgent> getRepository() {
        return knowledgeAgentMapper;
    }

    /**
     * 新增代理人
     * @param knowledgeAgentDto
     * @return
     */
    public String create(KnowledgeAgentDto knowledgeAgentDto) {
        KnowledgeAgent knowledgeAgent = new KnowledgeAgent();
        BeanUtils.copyProperties(knowledgeAgentDto,knowledgeAgent);
        knowledgeAgent.setUserNo(userService.queryLoginUsername());
        if (StringUtils.isEmpty(knowledgeAgent.getPkId())){
            //查询当前登陆人是否已经被代理了
            KnowledgeAgent knowledgeAgentOne = knowledgeAgentMapper.
                    selectOne(new LambdaQueryWrapper<KnowledgeAgent>().eq(KnowledgeAgent::getUserNo,knowledgeAgent.getUserNo()));
            if (knowledgeAgentOne != null){
                throw new RuntimeException("当前用户已经被他人代理,添加失败！");
            }
            knowledgeAgent.setPkId(IdGenerator.getIdStr());
            knowledgeAgentMapper.insert(knowledgeAgent);
        } else {
            knowledgeAgentMapper.updateById(knowledgeAgent);
        }
        return knowledgeAgent.getPkId();
    }

    /**
     * 查询代理人列表
     * @return
     */
    public KnowledgeAgentVo info() {
        String userNo = userService.queryLoginUsername();
        LambdaQueryWrapper<KnowledgeAgent> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(KnowledgeAgent::getUserNo,userNo);
        KnowledgeAgent knowledgeAgent = knowledgeAgentMapper.selectOne(wrapper);
        KnowledgeAgentVo knowledgeAgentVo = new KnowledgeAgentVo();
        if (knowledgeAgent == null){
            return knowledgeAgentVo;
        }
        BeanUtils.copyProperties(knowledgeAgent,knowledgeAgentVo);
        JPaasUser jPaasUser = userService.queryByUsername(knowledgeAgent.getTagerNo());
        knowledgeAgentVo.setTagerName(jPaasUser.getFullName());
        if (knowledgeAgent.getState() == 1){
            knowledgeAgentVo.setStateName("启用");
        } else if (knowledgeAgent.getState() == 2){
            knowledgeAgentVo.setStateName("禁用");
        }
        return knowledgeAgentVo;
    }


    /**
     * 清空代理人
     * @return
     */
    public String del() {
        String userNo = userService.queryLoginUsername();
        knowledgeAgentMapper.del(userNo);
        return "";
    }
}
