
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.KnowledgeApprovalNode;
import com.redxun.knowledge.entity.dao.KnowledgeApprovalNodeCategory;
import com.redxun.knowledge.entity.dao.KnowledgeApprovalNodeFlower;
import com.redxun.knowledge.entity.dao.KnowledgeCategory;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeDto;
import com.redxun.knowledge.entity.dto.KnowledgeApprovalNodeFlowerDto;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalNodeFlowerVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalNodeVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalVo;
import com.redxun.knowledge.mapper.KnowledgeApprovalNodeCategoryMapper;
import com.redxun.knowledge.mapper.KnowledgeApprovalNodeFlowerMapper;
import com.redxun.knowledge.mapper.KnowledgeApprovalNodeMapper;
import com.redxun.knowledge.mapper.KnowledgeCategoryMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.xml.bind.annotation.XmlType;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * [业务--知识审批节点表]业务服务类
 */
@Service
public class KnowledgeApprovalNodeServiceImpl extends SuperServiceImpl<KnowledgeApprovalNodeMapper, KnowledgeApprovalNode> implements BaseService<KnowledgeApprovalNode> {

    public static final String flowerName = "系统默认流程";

    @Resource
    private KnowledgeApprovalNodeMapper knowledgeApprovalNodeMapper;

    @Autowired
    private KnowledgeApprovalNodeFlowerMapper knowledgeApprovalNodeFlowerMapper;

    @Autowired
    private KnowledgeApprovalNodeCategoryMapper knowledgeApprovalNodeCategoryMapper;

    @Autowired
    private KnowledgeCategoryMapper knowledgeCategoryMapper;

    @Autowired
    private UserService userService;

    @Override
    public BaseDao<KnowledgeApprovalNode> getRepository() {
        return knowledgeApprovalNodeMapper;
    }

    public List<KnowledgeApprovalVo> queryApprovalNodes() {
        return knowledgeApprovalNodeMapper.queryApprovalNodes();
    }

    @Override
    public boolean saveOrUpdateBatch(Collection<KnowledgeApprovalNode> entityList, int batchSize) {
        return super.saveOrUpdateBatch(entityList, batchSize);
    }

    @Override
    public int insert(KnowledgeApprovalNode entity) {
        return 0;
    }

    /**
     * 分页查询审核流程节点
     *
     * @param filter
     * @return
     */
    public IPage findAllApprovalNodes(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = knowledgeApprovalNodeMapper.findAllApprovalNodes(filter.getPage(), filter.getParams(), params);
        return page;
    }

    public void deleteKnowledgeApprovalNode() {
        knowledgeApprovalNodeMapper.deleteKnowledgeApprovalNode();
    }

    /**
     * 添加审核流程(新)
     *
     * @param knowledgeApprovalNodeFlowerDto
     */
    @Transactional(rollbackFor = Exception.class)
    public String insertApprovalNodes(KnowledgeApprovalNodeFlowerDto knowledgeApprovalNodeFlowerDto) throws Exception {
        //修改(清除原本数据)
        if (StringUtils.isNotEmpty(knowledgeApprovalNodeFlowerDto.getPkId())) {
            knowledgeApprovalNodeMapper.deleteByFlowerId(knowledgeApprovalNodeFlowerDto.getPkId());
            knowledgeApprovalNodeCategoryMapper.deleteByFlowerId(knowledgeApprovalNodeFlowerDto.getPkId());
        }

        //判断输入的分类是否存在流程
        knowledgeApprovalNodeFlowerDto.getCategoryId().forEach(e -> {
            KnowledgeApprovalNodeCategory knowledgeApprovalNodeCategory = knowledgeApprovalNodeCategoryMapper.
                    selectOne(new LambdaQueryWrapper<KnowledgeApprovalNodeCategory>().eq(KnowledgeApprovalNodeCategory::getCategoryId, e));
            if (!Objects.isNull(knowledgeApprovalNodeCategory)) {
                KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(knowledgeApprovalNodeCategory.getCategoryId());
                if (!Objects.isNull(knowledgeCategory)) {
                    throw new RuntimeException("添加失败,关联的分类" + knowledgeCategory.getName() + "已有流程");
                } else {
                    throw new RuntimeException("添加失败,关联的分类不存在");
                }
            }
        });


        //如果名称是默认审核流程,关联分类为默认值
        if (knowledgeApprovalNodeFlowerDto.getFlowerName().equals(flowerName)) {
            Set<String> list = new HashSet<>();
            list.add("-2");
            knowledgeApprovalNodeFlowerDto.setCategoryId(list);
        }
        KnowledgeApprovalNodeFlower knowledgeApprovalNodeFlower = new KnowledgeApprovalNodeFlower();
        knowledgeApprovalNodeFlower.setFlowerName(knowledgeApprovalNodeFlowerDto.getFlowerName());
        if (StringUtils.isEmpty(knowledgeApprovalNodeFlowerDto.getPkId())) {
            knowledgeApprovalNodeFlowerMapper.insert(knowledgeApprovalNodeFlower);
        } else {
            knowledgeApprovalNodeFlower.setPkId(knowledgeApprovalNodeFlowerDto.getPkId());
            knowledgeApprovalNodeFlowerMapper.updateById(knowledgeApprovalNodeFlower);
        }

        //关联分类
        if (!CollectionUtils.isEmpty(knowledgeApprovalNodeFlowerDto.getCategoryId())) {
            knowledgeApprovalNodeFlowerDto.getCategoryId().forEach(e -> {
                KnowledgeApprovalNodeCategory knowledgeApprovalNodeCategory = new KnowledgeApprovalNodeCategory();
                knowledgeApprovalNodeCategory.setCategoryId(e);
                knowledgeApprovalNodeCategory.setFlowerId(knowledgeApprovalNodeFlower.getPkId());
                List<String> pathList = knowledgeCategoryMapper.selectPath(e);
                knowledgeApprovalNodeCategory.setPath(String.join(".", pathList));
                knowledgeApprovalNodeCategoryMapper.insert(knowledgeApprovalNodeCategory);
            });
        }
        //关联节点
        List<KnowledgeApprovalNodeDto> knowledgeApprovalNodes = knowledgeApprovalNodeFlowerDto.getKnowledgeApprovalNode();
        if (!CollectionUtils.isEmpty(knowledgeApprovalNodes)) {
            AtomicInteger sort = new AtomicInteger(1);
            knowledgeApprovalNodes.forEach(e -> {
                KnowledgeApprovalNode knowledgeApprovalNode = new KnowledgeApprovalNode();
                BeanUtils.copyProperties(e, knowledgeApprovalNode);
                knowledgeApprovalNode.setFlowerId(knowledgeApprovalNodeFlower.getPkId());
                knowledgeApprovalNode.setSort(sort.getAndIncrement());
                knowledgeApprovalNodeMapper.insert(knowledgeApprovalNode);
            });
        }
        return knowledgeApprovalNodeFlower.getPkId();
    }

    /**
     * 查询全部节点
     *
     * @return
     */
    public List<KnowledgeApprovalNodeFlowerVo> findAllNodes() {
        List<KnowledgeApprovalNodeFlowerVo> result = knowledgeApprovalNodeFlowerMapper.findAllNodes();
        result.forEach(e -> {
            if (flowerName.equals(e.getFlowerName())) {
                e.setCategory(new ArrayList<>());
            }
            List<String> category = new ArrayList<>();
            e.getCategory().forEach(h -> {
                KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(h);
                if (!Objects.isNull(knowledgeCategory)) {
                    category.add(knowledgeCategory.getName());
                }
            });
            e.setCategoryName(category);
        });
        return result;
    }

    /**
     * pc端查询审核流程
     *
     * @param categoryId
     * @return
     */
    public List<KnowledgeApprovalNodeVo> findAllNodesByCategoryId(String categoryId,String userId) {
        List<KnowledgeApprovalNodeVo> result = new ArrayList<>();
        //没输入分类id，查看默认流程后返回
        if (StringUtils.isEmpty(categoryId)) {
            categoryId = "-2";
            result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId(categoryId);
            defaultNode(result.get(0),userId);
            return result;
        }
        //输入分类id
        result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId(categoryId);
        //查询其上级是否有审核流程
        if (CollectionUtils.isEmpty(result)) {
            KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(categoryId);
            while (true) {
                if ("0".equals(knowledgeCategory.getPkId())) {
                    break;
                }
                //父级有审核模板，退出循环
                knowledgeCategory = knowledgeCategoryMapper.
                        selectOne(new LambdaQueryWrapper<KnowledgeCategory>().eq(KnowledgeCategory::getPkId, knowledgeCategory.getParent()));
                result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId(knowledgeCategory.getPkId());
                if (!CollectionUtils.isEmpty(result)) {
                    break;
                }
            }
        }
        //该分类没有审核流程,使用默认流程
        if (CollectionUtils.isEmpty(result)) {
            result = knowledgeApprovalNodeFlowerMapper.findAllNodesByCategoryId("-2");
        }
        defaultNode(result.get(0),userId);
        return result;
    }

    private void defaultNode(KnowledgeApprovalNodeVo result,String userId) {
        if (result.getIsFinal() != -1) {
            throw new RuntimeException("没找到默认节点");
        }
        //处理默认节点
        String deptId = null;
        if (StringUtils.isEmpty(userId)){
            deptId = userService.queryLoginUserDeptId();
        } else {
            deptId = userService.queryOsUserDto(userId).getDeptId();
        }

        String parentUserIds = knowledgeApprovalNodeFlowerMapper.findParentUserIdList(deptId);
        if (!StringUtils.isEmpty(parentUserIds)) {
            List<String> parentUserIdList = Arrays.asList(parentUserIds.split(","));
            //审核人userId(员工工号)
            String approverId = parentUserIdList.parallelStream().map(e -> userService.queryOsUserDto(e).getUserNo()).
                    collect(Collectors.joining(","));
            result.setApproverId(approverId);
            //审核人姓名
            String approverName = parentUserIdList.parallelStream().map(e -> userService.queryUser(e).getFullName()).
                    collect(Collectors.joining(","));
            result.setApproverName(approverName);
        } else {
            if (StringUtils.isEmpty(userId)){
                result.setApproverId(userService.queryLoginUsername());
                result.setApproverName(userService.queryLoginFullName());
            } else {
                result.setApproverId(userService.queryOsUserDto(userId).getUserNo());
                result.setApproverName(userService.queryUser(userId).getFullName());
            }

        }
    }

    /**
     * 删除审核流程
     *
     * @param pkId
     */
    @Transactional(rollbackFor = Exception.class)
    public void delApprovalNode(String pkId) {
        knowledgeApprovalNodeFlowerMapper.deleteByPkId(pkId);
        knowledgeApprovalNodeMapper.deleteByFlowerId(pkId);
        knowledgeApprovalNodeCategoryMapper.deleteByFlowerId(pkId);
    }

    /**
     * 查询选过的关联分类id集
     * @param pkId
     * @return
     */
    public List<String> findAllCategorySelected(String pkId) {
        List<String> idList = knowledgeApprovalNodeCategoryMapper.findAllCategorySelected(pkId);
        List<String> result = idList.stream().filter(e -> !("-2".equals(e))).collect(Collectors.toList());
        return result;
    }
}
